@@ -15,8 +15,6 @@
 
 """The Tornado web server and tools."""
 
-from __future__ import absolute_import, division, print_function
-
 # version is a human-readable version number.
 
 # version_info is a four-tuple for programmatic comparison. The first
@@ -24,5 +22,5 @@ from __future__ import absolute_import, division, print_function
 # is zero for an official release, positive for a development branch,
 # or negative for a release candidate or beta (after the base version
 # number has been incremented)
-version = "5.0.1"
-version_info = (5, 0, 1, 0)
+version = "6.0"
+version_info = (6, 0, 0, 0)
